searchData={"items":[{"type":"module","title":"Permit.Ecto","doc":"Integrates [`Permit`](https://github.com/curiosum-dev/permit) with Ecto, providing means to convert permissions to Ecto queries,\nautomatically constructing `Ecto.Query` scopes to preload records that meet authorization criteria.","ref":"Permit.Ecto.html"},{"type":"module","title":"Dependencies and related libraries - Permit.Ecto","doc":"`Permit.Ecto` depends on `Permit`. It can be used to build custom integrations or in conjunction with `Permit.Phoenix`, which uses\nthe generated `accessible_by/4` functions to automatically preload, authorize and inject records loaded via Ecto into\ncontroller assigns (see more in [`Permit.Phoenix documentation`](https://github.com/curiosum-dev/permit_phoenix)).","ref":"Permit.Ecto.html#module-dependencies-and-related-libraries"},{"type":"module","title":"Configuration - Permit.Ecto","doc":"```elixir\ndefmodule MyApp.Authorization do\n  use Permit.Ecto,\n    permissions_module: MyApp.Permissions,\n    repo: MyApp.Repo\nend\n\ndefmodule MyApp.Permissions do\n  use Permit.Ecto.Permissions, actions_module: Permit.Actions.CrudActions\n\n  def can(%{role: :admin} = user) do\n    permit()\n    |> all(MyApp.Blog.Article)\n  end\n\n  def can(%{id: user_id} = user) do\n    permit()\n    |> all(MyApp.Blog.Article, author_id: user_id)\n    |> read(MyApp.Blog.Article)\n  end\n\n  def can(user), do: permit()\nend\n\niex> MyApp.Repo.all(MyApp.Blog.Article)\n[\n  %MyApp.Blog.Article{id: 1, author_id: 1},\n  %MyApp.Blog.Article{id: 2, author_id: 1},\n  %MyApp.Blog.Article{id: 3, author_id: 2}\n]\n\n# The `accessible_by!/3` function also has a `accessible_by/3` variant which returns `{:ok, ...}` tuples.\n\niex> MyApp.Permissions.accessible_by!(%MyApp.Users.User{id: 1}, :update, MyApp.Blog.Article) |> MyApp.Repo.all()\n[%MyApp.Blog.Article{id: 1, ...}, %MyApp.Blog.Article{id: 2, ...}]\n\niex> MyApp.Permissions.accessible_by!(%MyApp.Users.User{id: 1}, :read, MyApp.Blog.Article) |> MyApp.Repo.all()\n[%MyApp.Blog.Article{id: 1, ...}, %MyApp.Blog.Article{id: 2, ...}, %MyApp.Blog.Article{id: 3, ...}]\n\niex> MyApp.Permissions.accessible_by!(%MyApp.Users.User{id: 3, role: :admin}, :update, MyApp.Blog.Article) |> MyApp.Repo.all()\n[%MyApp.Blog.Article{id: 1, ...}, %MyApp.Blog.Article{id: 2, ...}, %MyApp.Blog.Article{id: 3, ...}]\n```","ref":"Permit.Ecto.html#module-configuration"},{"type":"module","title":"Permit.Ecto.Permissions","doc":"Defines the application's permission set. Replaces `Permit.Permissions` when\n`Permit.Ecto` is used, but its syntax is identical.","ref":"Permit.Ecto.Permissions.html"},{"type":"module","title":"Example - Permit.Ecto.Permissions","doc":"```\ndefmodule MyApp.Permissions do\n  use Permit.Permissions, actions_module: Permit.Actions.CrudActions\n\n  @impl true\n  def can(%MyApp.User{role: :admin}) do\n    permit()\n    |> all(Article)\n  end\n\n  def can(%MyApp.User{id: user_id}) do\n    permit()\n    |> read(Article)\n    |> all(Article, author_id: user_id)\n  end\n\n  def can(_), do: permit()\nend\n```","ref":"Permit.Ecto.Permissions.html#module-example"},{"type":"module","title":"Associations - Permit.Ecto.Permissions","doc":"Conditions can be also defined for values of columns of associated records in `belongs_to`,\n`has_one` and `has_many` associations. Generated queries will automatically include appropriate\njoins for associated tables recursively.","ref":"Permit.Ecto.Permissions.html#module-associations"},{"type":"module","title":"Example - Permit.Ecto.Permissions","doc":"```\ndef can(user) do\n  permit()\n  |> read(Article, reviews: [approved: true]) # has_many association - any review is approved\n  |> read(Article, settings: [visible: true]) # has_one association - if settings.visible is true\n  |> read(Article, author: [region: [code: user.region_code]]) # belongs_to association, recursive\nend\n```","ref":"Permit.Ecto.Permissions.html#module-example-1"},{"type":"module","title":"Condition conversion - Permit.Ecto.Permissions","doc":"Conditions defined using standard operators such as equality, inequality, greater-than, less-than,\nLIKE and ILIKE are converted automatically (see `Permit.Operators`).\n\nOther conditions, such as those given as functions,\n\nRefer to `Permit.Permissions` documentation for more examples of usage.","ref":"Permit.Ecto.Permissions.html#module-condition-conversion"},{"type":"function","title":"Permit.Ecto.Permissions.construct_query/6","doc":"","ref":"Permit.Ecto.Permissions.html#construct_query/6"},{"type":"module","title":"Permit.Ecto.Permissions.DynamicQueryJoiner","doc":"Joins a DNF of conditions represented by a `Permit.Permissions.DisjunctiveNormalForm`\ninto an Ecto dynamic query.\n\nPart of the private API, subject to changes and not to be used on the\napplication level.","ref":"Permit.Ecto.Permissions.DynamicQueryJoiner.html"},{"type":"function","title":"Permit.Ecto.Permissions.DynamicQueryJoiner.add_joins/2","doc":"","ref":"Permit.Ecto.Permissions.DynamicQueryJoiner.html#add_joins/2"},{"type":"function","title":"Permit.Ecto.Permissions.DynamicQueryJoiner.to_dynamic_query/4","doc":"","ref":"Permit.Ecto.Permissions.DynamicQueryJoiner.html#to_dynamic_query/4"},{"type":"module","title":"Permit.Ecto.Permissions.ParsedCondition","doc":"Represents the product of parsing a condition by a function implementing\nthe `c:Permit.Permissions.can/1` callback.\n\nReplaces `Permit.Permissions.ParsedCondition` in applications using `Permit.Ecto`.\nRefer to `Permit.Permissions.ParsedCondition` documentation for more details.\n\nIn add_conditionition to the original implementation, its metadata also includes\ndynamic query constructors, derived from `Permit.Operators.DynamicQuery`.\n\nA condition parsed by Permit's rule syntax parser contains:\n* condition semantics, that is: a function that allows for checking\n  whether the condition is satisfied\n* an indication of whether it is negated (i.e. a condition defined as\n  `{:not, ...}`)\n* metadata (`:private`), which can be used by alternative parsers (e.g.\n  `Permit.Ecto.Permissions` puts dynamic query constructors there)\n\nPart of the private API, subject to changes and not to be used on the\napplication level.","ref":"Permit.Ecto.Permissions.ParsedCondition.html"},{"type":"type","title":"Permit.Ecto.Permissions.ParsedCondition.dynamic_query/0","doc":"","ref":"Permit.Ecto.Permissions.ParsedCondition.html#t:dynamic_query/0"},{"type":"function","title":"Permit.Ecto.Permissions.ParsedCondition.to_dynamic_query/4","doc":"","ref":"Permit.Ecto.Permissions.ParsedCondition.html#to_dynamic_query/4"},{"type":"behaviour","title":"Permit.Operators.DynamicQuery","doc":"Implemented to define a dynamic query builder function for an operator, that is\na module that implements `Permit.Operators.GenOperator`.\n\nFor example, when an operator is defined in the `Permit.Operators.Eq` module, its\ndynamic query builder function should be defined in `Permit.Operators.Eq.DynamicQuery`.\n\nPart of the private API, subject to changes and not to be used on the application level.","ref":"Permit.Operators.DynamicQuery.html"},{"type":"callback","title":"Permit.Operators.DynamicQuery.dynamic_query_fn/2","doc":"","ref":"Permit.Operators.DynamicQuery.html#c:dynamic_query_fn/2"},{"type":"module","title":"Permit.Ecto.Resolver","doc":"Implementation of `Permit.ResolverBase` behaviour, resolving and checks authorization of records or lists of records based on automatic Ecto query construction, taking parameters as input and `:base_query` and `:finalize_query` functions as means to transform the query based on e.g. current controller context.\n\nFor a resolver implementation not using Ecto for fetching resources, see `Permit.Resolver` from the `permit` library.\n\nThe usage of `Permit.Ecto.Resolver` as opposed to `Permit.Resolver` in `permit_ecto` library occurs because in the `m:Permit.Ecto.__using__/1` macro the `resolver_module/0` function is overridden to point to `Permit.Ecto.Resolver`.\n\nThis module is to be considered a private API of the authorization framework.\nIt should not be directly used by application code, but rather by wrappers\nproviding integration with e.g. Plug or LiveView.","ref":"Permit.Ecto.Resolver.html"},{"type":"function","title":"Permit.Ecto.Resolver.authorize_and_preload_all!/5","doc":"","ref":"Permit.Ecto.Resolver.html#authorize_and_preload_all!/5"},{"type":"function","title":"Permit.Ecto.Resolver.authorize_and_preload_one!/5","doc":"","ref":"Permit.Ecto.Resolver.html#authorize_and_preload_one!/5"},{"type":"function","title":"Permit.Ecto.Resolver.authorized?/4","doc":"","ref":"Permit.Ecto.Resolver.html#authorized?/4"},{"type":"module","title":"Permit.Ecto.Types","doc":"Defines Ecto-specific types for usage with Permit.","ref":"Permit.Ecto.Types.html"},{"type":"type","title":"Permit.Ecto.Types.base_query/0","doc":"Allows defining a base Ecto query based on current resolution context, e.g. query\nparameters, request URL or anything else (depending on execution context).","ref":"Permit.Ecto.Types.html#t:base_query/0"},{"type":"type","title":"Permit.Ecto.Types.finalize_query/0","doc":"Allows manipulating the query after it has been constructed by Permit's query builder,\nbut before it is executed by `Permit.Ecto.Resolver`.","ref":"Permit.Ecto.Types.html#t:finalize_query/0"},{"type":"module","title":"Permit.Ecto.Types.ConditionTypes","doc":"Provides new types for usage with Ecto queries, as well as  replacements for\ntypes initially defined in `Permit.Types.ConditionTypes`.","ref":"Permit.Ecto.Types.ConditionTypes.html"},{"type":"type","title":"Permit.Ecto.Types.ConditionTypes.condition/0","doc":"","ref":"Permit.Ecto.Types.ConditionTypes.html#t:condition/0"},{"type":"type","title":"Permit.Ecto.Types.ConditionTypes.condition_or_conditions/0","doc":"","ref":"Permit.Ecto.Types.ConditionTypes.html#t:condition_or_conditions/0"},{"type":"type","title":"Permit.Ecto.Types.ConditionTypes.fn1_condition_with_query/0","doc":"","ref":"Permit.Ecto.Types.ConditionTypes.html#t:fn1_condition_with_query/0"},{"type":"type","title":"Permit.Ecto.Types.ConditionTypes.fn2_condition_with_query/0","doc":"","ref":"Permit.Ecto.Types.ConditionTypes.html#t:fn2_condition_with_query/0"},{"type":"exception","title":"Permit.Ecto.UnconvertibleConditionError","doc":"","ref":"Permit.Ecto.UnconvertibleConditionError.html"},{"type":"exception","title":"Permit.Ecto.UndefinedConditionError","doc":"Raised when the permissions module (implementing `Permit.Permissions`) does not contain sufficient\ninformation to compute authorization for a given action.","ref":"Permit.Ecto.UndefinedConditionError.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}